"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaClass = void 0;
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.
var SchemaType_1 = require("./SchemaType");
var SchemaProperty_1 = require("./SchemaProperty");
var SchemaClass = /** @class */ (function (_super) {
    __extends(SchemaClass, _super);
    function SchemaClass(sourceObj) {
        var _this = _super.call(this, sourceObj) || this;
        _this._properties = new Map();
        _this._isAbstract = false;
        _this._extends = [];
        _this._implementations = [];
        if (sourceObj.properties) {
            for (var key in sourceObj.properties) {
                _this._properties.set(key, new SchemaProperty_1.SchemaProperty(key, sourceObj.properties[key]));
            }
        }
        if (sourceObj.isAbstract) {
            _this._isAbstract = true;
        }
        return _this;
    }
    Object.defineProperty(SchemaClass.prototype, "properties", {
        get: function () {
            return this._properties;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SchemaClass.prototype, "isAbstract", {
        get: function () {
            return this._isAbstract;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SchemaClass.prototype, "extends", {
        get: function () {
            return this._extends;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SchemaClass.prototype, "implementations", {
        get: function () {
            return this._implementations;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SchemaClass.prototype, "shorthand", {
        get: function () {
            return this._shorthand;
        },
        enumerable: false,
        configurable: true
    });
    // Gets all extended, including descendants
    SchemaClass.prototype.getAllExtended = function () {
        var answer = [];
        this.extends.forEach(function (extended) {
            answer.push(extended);
            extended.getAllExtended().forEach(function (descendantExtended) {
                answer.push(descendantExtended);
            });
        });
        return answer;
    };
    SchemaClass.prototype.getAllExtendedProperties = function () {
        var answer = [];
        this.getAllExtended().forEach(function (extended) {
            extended.properties.forEach(function (value, key) {
                answer.push(key);
            });
        });
        return answer;
    };
    // Gets all properties, including inherited properties
    SchemaClass.prototype.getAllProperties = function () {
        var answer = new Map();
        this.properties.forEach(function (value, key) {
            answer.set(key, value);
        });
        this.getAllExtended().forEach(function (extended) {
            extended.properties.forEach(function (value, key) {
                if (answer.has(key)) {
                    // If the property didn't have override on it, it gets overwritten by base property
                    if (!answer.get(key).override) {
                        answer.set(key, value);
                    }
                }
                else {
                    answer.set(key, value);
                }
            });
        });
        return answer;
    };
    SchemaClass.prototype.resolve = function (types) {
        var _this = this;
        // Resolve extends
        if (this.original.extends) {
            this.original.extends.split(",").forEach(function (extended) {
                extended = extended.trim();
                var foundType = types.get(extended);
                if (foundType == undefined) {
                    throw new Error("Class " + _this.type + " extends " + extended + ", but " + extended + " couldn't be found.");
                }
                _this.extends.push(foundType);
            });
        }
        // Resolve properties
        this.properties.forEach(function (prop) {
            prop.resolve(types);
        });
        if (this.original.shorthand) {
            var shorthandProp = this.properties.get(this.original.shorthand);
            if (shorthandProp === undefined) {
                throw new Error("Shorthand property " + this.original.shorthand + " wasn't found");
            }
            this._shorthand = shorthandProp;
        }
    };
    SchemaClass.prototype.resolveImplementations = function (types) {
        var _this = this;
        // This must be called AFTER resolve has been called, as it depends on extends
        types.forEach(function (type) {
            if (type instanceof SchemaClass) {
                if (type.extends.indexOf(_this) != -1) {
                    _this._implementations.push(type);
                }
            }
        });
    };
    return SchemaClass;
}(SchemaType_1.SchemaType));
exports.SchemaClass = SchemaClass;
//# sourceMappingURL=SchemaClass.js.map